var catw = require('../');
var test = require('tape');
var fs = require('fs');
var path = require('path');
var os = require('os');
var tmpdir = os.tmpdir || os.tmpDir;
var mkdirp = require('mkdirp');
var concat = require('concat-stream');
var through = require('through');

test('source transform', function (t) {
    t.plan(4);
    var dir = path.join(tmpdir(), 'cat-watch-test-exec', Math.random()+'');
    mkdirp.sync(dir);
    
    fs.writeFileSync(path.join(dir, 'beep.txt'), 'beep');
    fs.writeFileSync(path.join(dir, 'boop.txt'), 'boop');
    
    var cat = catw(path.join(dir, '*.txt'), {
        transform: function (file) {
            if (path.basename(file) !== 'beep.txt') return through();
            return through(write);
            function write (buf) {
                this.queue(buf.toString('utf8').toUpperCase());
            }
        }
    });
    t.on('end', function () { cat.close() });
    var expected = [ 'BEEPboop', 'BEEP boop', 'BEEP boop!', 'BEEP boop!!!' ];
    
    cat.on('stream', function (stream) {
        stream.pipe(concat(function (body) {
            t.equal(body.toString('utf8'), expected.shift());
            setTimeout(nextUpdate, 100);
        }));
    });
    
    var updates = [
        [ 'beep.txt', 'beep ' ],
        [ 'boop.txt', 'boop!' ],
        [ 'c.txt', '!!' ]
    ];
    
    function nextUpdate () {
        if (updates.length === 0) return;
        var file = updates.shift();
        fs.writeFile(path.join(dir, file[0]), file[1], function (err) {
            if (err) return t.fail(err)
        });
    }
});
